@@ -1,28 +1,36 @@
-/* regexprops.c -- document the properties of the regular expressions 
+/* regexprops.c -- document the properties of the regular expressions
                    understood by gnulib.
 
-   Copyright 2005 Free Software Foundation, Inc.
+   Copyright 2005, 2007 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+
+/*
+   The output of this program is included in the GNU findutils source
+   distribution.  The copying conditions for that file are generated
+   by the copying() function below.
+*/
+
 /* Written by James Youngman, <jay@gnu.org>. */
 
 #include <config.h>
 
 
 #include <stdio.h>
+#include <string.h>
 #include <unistd.h>
 #include <errno.h>
 
@@ -36,7 +44,7 @@ char *program_name;
 static void output(const char *s, int escape)
 {
   (void) escape;
-  
+
   fputs(s, stdout);
 }
 
@@ -61,6 +69,13 @@ static void directive(const char *s)
   output(s, 0);
 }
 
+static void comment(const char *s)
+{
+  directive("@c ");
+  literal(s);
+  newline();
+}
+
 static void enum_item(const char *s)
 {
   newline();
@@ -77,14 +92,14 @@ static void begin_subsection(const char *name,
   (void) next;
   (void) prev;
   (void) up;
-  
+
   newline();
-  
+
   directive("@node ");
   content(name);
   content(" regular expression syntax");
   newline();
-  
+
   directive("@subsection ");
   output("@samp{", 0);
   content(name);
@@ -148,7 +163,7 @@ describe_regex_syntax(int options)
     }
   content(".  ");
   newpara();
-  
+
   if (!(options & RE_LIMITED_OPS))
     {
       begintable_markup("@samp");
@@ -178,7 +193,7 @@ describe_regex_syntax(int options)
 	}
       endtable();
     }
-  
+
   newpara();
 
   content("Bracket expressions are used to match ranges of characters.  ");
@@ -188,7 +203,7 @@ describe_regex_syntax(int options)
   else
     content("ignored");
   content(".  ");
-  
+
   if (options &  RE_BACKSLASH_ESCAPE_IN_LISTS)
     literal("Within square brackets, @samp{\\} can be used to quote "
 	    "the following character.  ");
@@ -236,7 +251,7 @@ describe_regex_syntax(int options)
   if (options & RE_NO_BK_PARENS)
     {
       literal("Grouping is performed with parentheses @samp{()}.  ");
-      
+
       if (options & RE_UNMATCHED_RIGHT_PAREN_ORD)
 	literal("An unmatched @samp{)} matches just itself.  ");
     }
@@ -244,7 +259,7 @@ describe_regex_syntax(int options)
     {
       literal("Grouping is performed with backslashes followed by parentheses @samp{\\(}, @samp{\\)}.  ");
     }
-  
+
   if (options & RE_NO_BK_REFS)
     {
       content("A backslash followed by a digit matches that digit.  ");
@@ -258,7 +273,7 @@ describe_regex_syntax(int options)
 	literal("@samp{\\(}");
       content(".  ");
     }
-  
+
 
   newpara();
   if (!(options & RE_LIMITED_OPS))
@@ -293,7 +308,7 @@ describe_regex_syntax(int options)
 	{
 	  if (options & RE_NEWLINE_ALT)
 	    enum_item("After a newline");
-	  
+
 	  if (options & RE_NO_BK_VBAR )
 	    enum_item("After the alternation operator @samp{|}");
 	  else
@@ -318,7 +333,7 @@ describe_regex_syntax(int options)
 	{
 	  if (options & RE_NEWLINE_ALT)
 	    enum_item("Before a newline");
-	  
+
 	  if (options & RE_NO_BK_VBAR)
 	    enum_item("Before the alternation operator @samp{|}");
 	  else
@@ -340,7 +355,7 @@ describe_regex_syntax(int options)
 	    literal("@samp{\\*}, @samp{\\+} and @samp{\\?} ");
 	  else
 	    literal("@samp{*}, @samp{+} and @samp{?} ");
-	  
+
 	  if (options & RE_CONTEXT_INVALID_OPS)
 	    {
 	      content("are special at any point in a regular expression except the following places, where they are not allowed:");
@@ -349,7 +364,7 @@ describe_regex_syntax(int options)
 	    {
 	      content("are special at any point in a regular expression except:");
 	    }
-	  
+
 	  beginenum();
 	  enum_item("At the beginning of a regular expression");
 	  enum_item("After an open-group, signified by ");
@@ -365,7 +380,7 @@ describe_regex_syntax(int options)
 	    {
 	      if (options & RE_NEWLINE_ALT)
 		enum_item("After a newline");
-	      
+
 	      if (options & RE_NO_BK_VBAR)
 		enum_item("After the alternation operator @samp{|}");
 	      else
@@ -374,10 +389,10 @@ describe_regex_syntax(int options)
 	  endenum();
 	}
     }
-  
+
 
   newpara();
-  if (options & RE_INTERVALS) 
+  if (options & RE_INTERVALS)
     {
       if (options & RE_NO_BK_BRACES)
 	{
@@ -403,7 +418,7 @@ describe_regex_syntax(int options)
 	      literal("Invalid intervals such as @samp{a\\@{1z} are not accepted.  ");
 	    }
 	}
-      
+
     }
 
   newpara();
@@ -419,46 +434,113 @@ describe_regex_syntax(int options)
 }
 
 
+static void copying(void)
+{
+  static const char *copy_para[]=
+    {
+      "Copyright (C) 1994, 1996, 1998, 2000, 2001, 2003, 2004, 2005, 2006, 2007"
+      ,"Free Software Foundation, Inc."
+      ,""
+      ,"Permission is granted to copy, distribute and/or modify this document"
+      ,"under the terms of the GNU Free Documentation License, Version 1.2 or"
+      ,"any later version published by the Free Software Foundation; with no"
+      ,"Invariant Sections, with no Front-Cover Texts, and with no Back-Cover"
+      ,"Texts.  A copy of the license is included in the ``GNU Free"
+      ,"Documentation License'' file as part of this distribution."
+      ""
+      ,NULL
+    };
+  const char **s = copy_para;
+  while (*s)
+    comment(*s++);
+}
+
+static int
+ignore(int ix, const unsigned int context)
+{
+  return 0 == (get_regex_type_context(ix) & context);
+}
 
 static void
-menu(void)
+menu(unsigned int context)
 {
   int i, options;
   const char *name;
-  
+
   output("@menu\n", 0);
   for (i=0;
        options = get_regex_type_flags(i),
 	 name=get_regex_type_name(i);
        ++i)
     {
-      output("* ", 0);
-      output(name, 0);
-      content(" regular expression syntax");
-      output("::", 0);
-      newline();
+      if (!ignore(i, context))
+	{
+	  output("* ", 0);
+	  output(name, 0);
+	  content(" regular expression syntax");
+	  output("::", 0);
+	  newline();
+	}
     }
   output("@end menu\n", 0);
 }
 
 
+
+static const char *
+get_next(unsigned int ix, unsigned int context)
+{
+  const char *next;
+  while (get_regex_type_name(ix))
+    {
+      if (!ignore(ix, context))
+	{
+	  next = get_regex_type_name(ix);
+	  if (NULL == next)
+	    return "";
+	  else
+	    return next;
+	}
+      ++ix;
+    }
+  return "";
+}
+
+
 static void
-describe_all(const char *up)
+describe_all(const char *contextname,
+	     unsigned int context,
+	     const char *up)
 {
   const char *name, *next, *previous;
   int options;
   int i, parent;
 
-  menu();
-  
+  copying();
+  newline();
+  literal("@c this regular expression description is for: ");
+  literal(contextname);
+  newline();
+  newline();
+  menu(context);
+
   previous = "";
-  
+
   for (i=0;
        options = get_regex_type_flags(i),
 	 name=get_regex_type_name(i);
        ++i)
     {
-      next = get_regex_type_name(i+1);
+      if (ignore(i, context))
+	{
+	  fprintf(stderr,
+		  "Skipping regexp type %s for context %s\n",
+		  name, contextname);
+	  name = previous;
+	  continue;
+	}
+
+      next = get_next(i+1, context);
       if (NULL == next)
 	next = "";
       begin_subsection(name, next, previous, up);
@@ -482,11 +564,31 @@ describe_all(const char *up)
 int main (int argc, char *argv[])
 {
   const char *up = "";
+  unsigned int context = CONTEXT_ALL;
+  const char *contextname = "all";
   program_name = argv[0];
-  
+
   if (argc > 1)
-    up = argv[1];
-  
-  describe_all(up);
+    {
+      up = argv[1];
+    }
+  if (argc > 2)
+    {
+      contextname = argv[2];
+      if (0 == strcmp(contextname, "findutils"))
+	context = CONTEXT_FINDUTILS;
+      else if (0 == strcmp(contextname, "generic"))
+	context = CONTEXT_GENERIC;
+      else if (0 == strcmp(contextname, "all"))
+	context = CONTEXT_ALL;
+      else
+	{
+	  fprintf(stderr, "Unexpected context %s",
+		  contextname);
+	  return 1;
+	}
+    }
+
+  describe_all(contextname, context, up);
   return 0;
 }
