@@ -1,10 +1,10 @@
 /* wait.h -- POSIX macros for evaluating exit statuses
    Copyright (C) 1990 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,9 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
-
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 #include <sys/types.h>		/* For pid_t. */
 #if HAVE_SYS_WAIT_H
 #include <sys/wait.h>
