@@ -1,5 +1,5 @@
 /* Test of exact or abbreviated match search.
-   Copyright (C) 1990, 1998-1999, 2001-2007 Free Software Foundation, Inc.
+   Copyright (C) 1990, 1998-1999, 2001-2008 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,10 +19,6 @@
 
 #include <config.h>
 
-/* Some packages define ARGMATCH_DIE in their <config.h>.  Here we want to
-   assume the default definition of ARGMATCH_DIE.  */
-#undef ARGMATCH_DIE
-
 #include "argmatch.h"
 
 #include <stdio.h>
@@ -36,11 +32,18 @@
       if (!(expr))							     \
         {								     \
           fprintf (stderr, "%s:%d: assertion failed\n", __FILE__, __LINE__); \
+          fflush (stderr);						     \
           abort ();							     \
         }								     \
     }									     \
   while (0)
 
+/* Some packages define ARGMATCH_DIE and ARGMATCH_DIE_DECL in <config.h>, and
+   thus must link with a definition of that function.  Provide it here.  */
+#ifdef ARGMATCH_DIE_DECL
+ARGMATCH_DIE_DECL { exit (1); }
+#endif
+
 enum backup_type
 {
   no_backups,
